<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform">
    <!-- 
      WI 18846:  We need to allow FOP to word wrap on characters like '_', '-', and '.'.
        
      This template will modify text nodes such that they will wrap on an underscore, hyphen, 
      and full-stop (period).  It is probably a good idea to invoke this anywhere you are putting 
      text into a table cell.
      -->
    <xsl:template match="text()">
        <xsl:call-template name="wrap-on-special-characters">
            <xsl:with-param name="text-content" select="."/>
        </xsl:call-template>
    </xsl:template>
    
    <!--
      This template can be used to allow a given string to word-wrap on certain non-whitespace characters.  It 
      works by inserting zero-width spaces after any occurances of these special characters.
      
      Characters that will be wrapped on:
      
      underscore      (_)                    back slash      (\)
      hyphen          (-)                    foward slash    (/)
      full stop       (.)
        
      text-content:  The string that needs to be able to wrap on special characters.
      -->
    <xsl:template name="wrap-on-special-characters">
        <xsl:param name="text-content"/>
        
        <xsl:variable name="underscore">               <xsl:text>_</xsl:text>            </xsl:variable>
        <xsl:variable name="wrapping-underscore">      <xsl:text>_&#x200B;</xsl:text>    </xsl:variable>
        <xsl:variable name="hyphen">                   <xsl:text>-</xsl:text>            </xsl:variable>
        <xsl:variable name="wrapping-hyphen">          <xsl:text>-&#x200B;</xsl:text>    </xsl:variable>
        <xsl:variable name="full-stop">                <xsl:text>.</xsl:text>            </xsl:variable>
        <xsl:variable name="wrapping-full-stop">       <xsl:text>.&#x200B;</xsl:text>    </xsl:variable>
        <xsl:variable name="back-slash">               <xsl:text>\</xsl:text>            </xsl:variable>
        <xsl:variable name="wrapping-back-slash">      <xsl:text>\&#x200B;</xsl:text>    </xsl:variable>
        <xsl:variable name="foward-slash">             <xsl:text>/</xsl:text>            </xsl:variable>
        <xsl:variable name="wrapping-foward-slash">    <xsl:text>/&#x200B;</xsl:text>    </xsl:variable>
        
        <!-- wraps on underscores -->
        <xsl:variable name="wrap-underscores">
            <xsl:call-template name="replaceAll">
                <xsl:with-param name="original" select="$text-content"/>
                <xsl:with-param name="find-string" select="$underscore"/>
                <xsl:with-param name="replace-string" select="$wrapping-underscore"/>
            </xsl:call-template>
        </xsl:variable>
        
        <!-- wraps on hyphens -->
        <xsl:variable name="wrap-hyphens">
            <xsl:call-template name="replaceAll">
                <xsl:with-param name="original" select="$wrap-underscores"/>
                <xsl:with-param name="find-string" select="$hyphen"/>
                <xsl:with-param name="replace-string" select="$wrapping-hyphen"/>
            </xsl:call-template>
        </xsl:variable>
        
        <!-- wraps on full stops (periods) -->
        <xsl:variable name="wrap-full-stops">
            <xsl:call-template name="replaceAll">
                <xsl:with-param name="original" select="$wrap-hyphens"/>
                <xsl:with-param name="find-string" select="$full-stop"/>
                <xsl:with-param name="replace-string" select="$wrapping-full-stop"/>
            </xsl:call-template>    
        </xsl:variable>
        
        <!-- wraps on back slashes -->
        <xsl:variable name="wrap-back-slashes">
            <xsl:call-template name="replaceAll">
                <xsl:with-param name="original" select="$wrap-full-stops"/>
                <xsl:with-param name="find-string" select="$back-slash"/>
                <xsl:with-param name="replace-string" select="$wrapping-back-slash"/>
            </xsl:call-template>
        </xsl:variable>
        
        <!-- wraps on foward slashes -->
        <xsl:call-template name="replaceAll">
            <xsl:with-param name="original" select="$wrap-back-slashes"/>
            <xsl:with-param name="find-string" select="$foward-slash"/>
            <xsl:with-param name="replace-string" select="$wrapping-foward-slash"/>
        </xsl:call-template>
        
    </xsl:template>
    
    <!-- 
      This function can be used to replace every occurance of some substring in a given string with 
      another given string.
        
      original:  The string to be searched and modified
      find-string:  The substring within original we are going to replace
      replace-string:  The substring that find-string will be replaced with
      -->
    <xsl:template name="replaceAll">
        <xsl:param name="original"/>
        <xsl:param name="find-string"/>
        <xsl:param name="replace-string"/>
        
        <xsl:choose>
            <xsl:when test="contains($original, $find-string)">
                <xsl:value-of select="substring-before($original, $find-string)"/>
                <xsl:value-of select="$replace-string"/>
                <xsl:call-template name="replaceAll">
                    <xsl:with-param name="original" select="substring-after($original, $find-string)"/>
                    <xsl:with-param name="find-string" select="$find-string"/>
                    <xsl:with-param name="replace-string" select="$replace-string"/>
                </xsl:call-template>
            </xsl:when>
            <xsl:otherwise>
                <xsl:value-of select="$original"/>
            </xsl:otherwise>
        </xsl:choose>
    </xsl:template>
    
    <!-- 
      This method requires a XSLT 2.0 processor so we can't use it right now.  When we do get a 
      XSLT 2.0 processor we can replace "insert-zero-width-space" and "replaceAll" with these 3 
      lines.
      
      <xsl:template match="text()">
          <xsl:value-of select="replace(., '[_-\.]', '$0&#x200B;')"/>
      </xsl:template>
      -->
    
    <!-- 
      This template can be used to wrap on any character in a string.  This is generally not a good 
      idea but in some cases might be necessary.
      
      text-content:  The string that needs to be able to wrap on any character. 
      -->
    <xsl:template name="wrap-on-all-characters">
        <xsl:param name="text-content"/>
        
        <xsl:if test="string-length($text-content)">
            <xsl:value-of select="substring($text-content,1,1)"/>
            <xsl:text>&#x200B;</xsl:text>
            <xsl:call-template name="wrap-on-all-characters">
                <xsl:with-param name="text-content" select="substring($text-content,2)"/>
            </xsl:call-template>
        </xsl:if>
    </xsl:template>
    
</xsl:stylesheet>