/******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008, 2010. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************
 *
 * Summary:
 *   A view layout subclass of ContentPane with layout as the following:
 * 
 *     +--------+------------------+---------------------------+--------------------+
 *     |  icon  | left header node |         right header node | right actions area |
 *     |  node  +------------------+---------------------------+--------------------+
 *     |        | type node / content selector                                      |
 *     +--------+--------+----------------------------------------------------------+
 *     | attributes node | status node with icon                                    |
 *     +-----------------+----------------------------------------------------------+
 *     | summary node                                                               |
 *     +----------------------------------------------------------------------------+
 *     | left actions area                                                          |
 *     +----------------------------------------------------------------------------+
 *     |                                                                            |
 *     | main / alt content container node                                          |
 *     |                                                                            |
 *     +----------------------------------------------------------------------------+
 *
 *   The left header are used for in-progress indicator and the title.  The title
 *   can be editable if the attribute "titleEditable" is set to true.  CS Help icon
 *   is located to the right of the title and only visible if the attribute
 *   "contextHelpId" is set.
 *   
 *   The icon take CSS class with a 48x48 pixels image.  It is only visible if the
 *   attribute "iconClass" is set.
 *   
 *   The type node can be used for content selector when dual contents is used. Set
 *   the attribute "useDualContents" to true to have a main or alternate content
 *   container.  Labels must be specified for the content selector for selected and
 *   deselected content container.
 *   
 *   Attributes node area is an opened DIV for consumer to populate.  There are CSS
 *   classes defined for comsumer to use to layout the nodes properly.  The classes
 *   are "attributes-entry", "attributes-entry-label", "attributes-entry-value" for
 *   text value, and "attributes-entry-icon" for icon value.
 *   
 *   Status node with icon is used for server call return status.  There are API's
 *   to set the status message and icon.  Only the status label of "Success...",
 *   "Error...", or "Info..." is shown and the detail message is displayed as a
 *   tooltip when hover over.  The node is hidden when the status is cleared.
 *   
 *   Summary node is another opened DIV for consumer to set the innerHTML for simple
 *   message or append children nodes.
 *   
 *   The left actions area uses action links with icons.  Use the API addLeftViewAction() to
 *   create the action in the area.  The container area is hidden until action(s) is added.
 *   
 *   The right actions area at the top right corner uses plain HTML button.  use the
 *   API addRightViewAction to create action in the area.
 *   
 */

dojo.provide("com.ibm.asq.common.web.ui.internal.view.layout.ViewHeaderLayout");

dojo.require("dijit._Templated");
dojo.require("dijit.layout.ContentPane");
dojo.require("dijit.InlineEditBox");
dojo.require("dijit.form.Button");
dojo.require("dijit.form.Textarea");
dojo.require("jazz.ui.HoverPopup");
dojo.require("dojox.data.dom");

dojo.require("com.ibm.asq.common.web.ui.internal.util.CSHelp");
dojo.require("com.ibm.asq.common.web.ui.internal.util.MessageUtil");
dojo.require("com.ibm.asq.common.web.ui.internal.view.common.ImageCache");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.InlineEditor");

(function() {

dojo.declare("HeaderLayout.PairButtons", null, {
    headerButton: null,
    bottomButton: null,
    isButtonWidget: false,
	id: null,
	
    constructor: function(headerButton, bottomButton, isButtonWidget){
        this.headerButton = headerButton;
        this.bottomButton = bottomButton;
		this.isButtonWidget = isButtonWidget;
    },
    
	addClass: function(className) {
		if(this.isButtonWidget) {
			dojo.addClass(this.headerButton.domNode, className);
			dojo.addClass(this.bottomButton.domNode, className);
		} else {
			dojo.addClass(this.headerButton, className);
			dojo.addClass(this.bottomButton, className);
		}
	},
	
	removeClass: function(className) {
		if(this.isButtonWidget) {
			dojo.removeClass(this.headerButton.domNode, className);
			dojo.removeClass(this.bottomButton.domNode, className);
		} else {
			dojo.removeClass(this.headerButton, className);
			dojo.removeClass(this.bottomButton, className);
		}
	},
	
	setClass: function(className) {
		if(this.isButtonWidget) {
			this.headerButton.domNode.className = className;
			this.bottomButton.domNode.className = className;
		} else {
			this.headerButton.className = className;
			this.bottomButton.className = className;
		}
	},
	
	hasClass: function(className) {
		if(this.isButtonWidget) {
			return dojo.hasClass(this.headerButton.domNode, className);
		} else {
			return dojo.hasClass(this.headerButton, className);
		}
	},
	
	addIconClass: function(className) {
		if(this.isButtonWidget) {
			dojo.addClass(this.headerButton.iconNode, className);
			dojo.addClass(this.bottomButton.iconNode, className);
		} 
	},
	
	removeIconClass: function(className) {
		if(this.isButtonWidget) {
			dojo.removeClass(this.headerButton.iconNode, className);
			dojo.removeClass(this.bottomButton.iconNode, className);
		} 
	},
	
	setDisabled: function(disable) {		
		if(this.isButtonWidget) {
			this.headerButton.attr('disabled', disable);
			this.bottomButton.attr('disabled', disable);
		} else {
			this.headerButton.disabled = disable;
			this.bottomButton.disabled = disable;
		}
        if ((this.headerButton.baseClass == "dijitDropDownButton") && this.headerButton.domNode && this.bottomButton.domNode) {
            dojo.toggleClass(this.headerButton.domNode, "view-action-dropdown-disabled", disable);
            dojo.toggleClass(this.bottomButton.domNode, "view-action-dropdown-disabled", disable);
            this.headerButton.disabled = disable;
            this.bottomButton.disabled = disable;
        }
	},
	
	isDisabled: function() {
		if(this.isButtonWidget) {
			return this.headerButton.isDisabled();
		} else {
			return this.headerButton.disabled;
		}
	},
	
	setId: function(id) {
		this.id = id;
	},
	
	getId: function() {
		return this.id;
	},
	
	setStyle: function(styleName, styleValue) {
		if(this.isButtonWidget) {
			dojo.style(this.headerButton.domNode, styleName, styleValue);
			dojo.style(this.bottomButton.domNode, styleName, styleValue);
		} else {
			dojo.style(this.headerButton, styleName, styleValue);
			dojo.style(this.bottomButton, styleName, styleValue);
		}
	},
	
	setLabel: function(label) {
		if(this.isButtonWidget) {
			this.headerButton.setLabel(label);
			this.bottomButton.setLabel(label);
		}
	}
});

dojo.declare("com.ibm.asq.common.web.ui.internal.view.layout.ViewHeaderLayout",
[dijit.layout.ContentPane, dijit._Templated],
{
    templateString: '<div id="${id}" style="display: inline" dojoAttachPoint="domNode" tabindex="-1" class="com-ibm-asq-common-web-ui-view-header">' +
                    '  <div dojoAttachPoint="viewHeaderBackNode" class="view-header-back" tabindex="-1">' +
                    '    <div class="view-header-container" tabindex="-1">' +
                    '      <div dojoAttachPoint="artifactContainer" class="artifact-container">' +
                    '        <div dojoAttachPoint="iconNode" class="${iconClass}" style="float:left"></div>' +
                    '        <div tabindex="-1" class="artifact-node">' +
                    '          <div>' +
                    '            <div dojoAttachPoint="actionsAreaRight" class="actions-area-right" style="float:right"></div>' +
                    '            <div dojoAttachPoint="actionsAreaLeft" class="actions-area-left" style="float:right"></div>' +
                    '            <div dojoAttachPoint="viewHeaderRight" waiRole="region" class="view-right-header" style="float:right"></div>' +
                    '            <div dojoAttachPoint="progressIndicator" class="progress-indicator" style="display: none">' +
                    '              <div class="view-progress-image">' +
                    '                <span dojoAttachPoint="progressLabel" class="view-progress-label"></span>' +
                    '              </div>' +
                    '            </div>' +
                    '            <div dojoAttachPoint="viewHeaderLeft" class="view-left-header">' +
                    '              <span dojoAttachPoint="viewTitle" class="view-title"></span>' +
                    '              <span dojoAttachPoint="viewSubTitle" class="view-title"></span>' +
                    '              <span dojoAttachPoint="titleContainer" style="width:100%;">' +
                    '                <span dojoAttachPoint="viewEditableTitle" tabIndex="1" dojoType="com.ibm.asq.common.web.ui.internal.widgets.InlineEditBox"' +
                    '                    value="${title}" class="view-title"></span>' +
                    '                <span dojoAttachPoint="viewHelpButtonNode" class="help-button-node"></span>' +
                    '              </span>' +
                    '            </div>' +
                    '          </div>' +
                    '          <div style="float:right;clear:right;width:auto;position:relative;" dojoAttachPoint="wholestatusContainer"> ' +
                    '            <span dojoAttachPoint="statusContainer">' +
                    '              <img dojoAttachPoint="statusSuccessNode" class="view-status-icon"></img>' +
                    '              <img dojoAttachPoint="statusErrorNode" class="view-status-icon"></img>' +
                    '              <img dojoAttachPoint="statusWarningNode" class="view-status-icon"></img>' +
                    '              <img dojoAttachPoint="statusInfoNode" class="view-status-icon"></img>' +
                    '              <span dojoAttachPoint="statusLineNode" tabIndex="0" class="view-status-value">&nbsp;</span>' +
                    '            </span>' +
                    '          </div> ' +
                    '          <div>' +
                    '            <span dojoAttachPoint="typeNode" class="view-artifact-type">artifact type</span>' +
                    '            <span dojoAttachPoint="contentSelectorNode" class="view-artifact-type">' +
                    '              <a href="javascript:{}" style="text-decoration:none"><span dojoAttachPoint="mainContentSelector" dojoAttachEvent="onclick:_selectMainContent,onkeypress:_onKeySelectMainContent"></span></a>' +
                    '              <span class="content-selector-separator">&nbsp;|&nbsp;</span>' +
                    '              <a href="javascript:{}" style="text-decoration:none"><span dojoAttachPoint="altContentSelector" dojoAttachEvent="onclick:_selectAltContent,onkeypress:_onKeySelectAltContent"></span></a>' +
                    '            </span>' +
                    '          </div>' +
                    '        </div>' +
                    '      </div>' +
                    '      <div class="attributes-node" dojoAttachPoint="attributesContainerNode">' +
                    '        <table>' +
                    '          <tr>' +
                    '            <td width="100%">' +
                    '              <div dojoAttachPoint="attributesNode"></div>' +
                    '            </td>' +
                    '            <td>' +
                    '              <div dojoAttachPoint="attributesNodeExt"></div>' +
                    '            </td>' +
                    '          </tr>' +
                    '        </table>' +
                    '      </div>' +
                    '      <div dojoAttachPoint="summaryNode" class="summary-node">' +
                    '        <span dojoAttachPoint="viewSummaryLabel" class="summary-label" style="float:left"></span>' +
                    '        <span class="attributes-entry-value"></span>' +
                    '        <span dojoAttachPoint="viewSummary" class="attributes-entry-value"></span>' +
                    '        <span dojoAttachPoint="viewEditableSummary" dojoType="com.ibm.asq.common.web.ui.internal.widgets.InlineEditArea"' +
                    '              class="summary-value">' +
                    '        </span>' +

 // for defect 21686.
 // Add a dummy text area to tickle inline editor's textarea. For some reason
 // having a staticly created textarea, the inline's editor's textarea works much
 // better. The initial delay that we see between the first clicked to the point
 // it can accept input is gone. However, this is just a quick workaround. 
 // We need to understand why this is the case...
 //                    
((dojo.isFF >= 3) ? '        <span dojoAttachPoint="dummyText" style="display:none;" dojoType="dijit.form.Textarea"></span>' : '') +
     
                    '      </div>' +
                    '    </div>' +
                    '  </div>' +
                    '  <div dojoAttachPoint="containerNode" waiRole="region" class="view-container"></div>' +
                    '  <div dojoAttachPoint="altContainerNode" waiRole="region" class="view-container" style="display:none"></div>' +
					'  <div dojoAttachPoint="viewHeaderBottomNode" style="display: none; width:99.5%;" tabindex="-1">' +
                    '      <div style="float:right;clear:right;width:auto;position:relative;">' +
                    '         <div dojoAttachPoint="actionsBottomAreaRight" class="actions-area-right" style="float:right; margin-right: 10px;"></div>' +
					'         <div dojoAttachPoint="actionsBottomAreaLeft" class="actions-area-left" style="float:right"></div>' +
                    '      </div>' +
                    '      <div style="float:right;width:auto;position:relative;" dojoAttachPoint="wholestatusBottomContainer"> ' +
                    '        <span dojoAttachPoint="statusBottomContainer" style="padding-right: 10px;">' +
                    '          <img dojoAttachPoint="statusSuccessBottomNode" class="view-status-icon"></img>' +
                    '          <img dojoAttachPoint="statusErrorBottomNode" class="view-status-icon"></img>' +
                    '          <img dojoAttachPoint="statusWarningBottomNode" class="view-status-icon"></img>' +
                    '          <img dojoAttachPoint="statusInfoBottomNode" class="view-status-icon"></img>' +
                    '          <span dojoAttachPoint="statusLineBottomNode" class="view-status-value">&nbsp;</span>' +
                    '        </span>' +
                    '      </div> ' +
                    '  </div>' +
                    '</div>',

    titleEditable: false,
    contextHelpId: null,
    csHelpEngine: null,
	cshelp: null,
    csHelpProviderId: null,
    iconClass: null,
    
    noTitleLabel: "",  // prompt label for editable title when no value is specified
    noSummaryLabel: "",  // prompt label for editable summary when no value is specified
    summaryAsDesc: true, // The summary node is mostly used for description so the no value prompt
                         // string is default to say description.  Set this to false to have the
                         // default no value prompt string to say summary.
    title: "",
	maxTitleLength: 250,  // Maximum number of characters that can be entered in the editable title in line edit box
	maxSummaryLength: 0,  // Maximum number of characters that can be entered in the summary, 0 for no limit
	subTitle: "",
    typeLabel: "",
    summaryEditable: false,
    summaryLabel: "",
    summary: "",
    noline: false,
    
    useDualContents: false,
    defaultMainContent: true,
    // consumer should specify these labels
    //
    mainContentSelectedLabel: "",
    mainContentDeselectedLabel: "",
    altContentSelectedLabel: "",
    altContentDeselectedLabel: "",
    
    widgetsInTemplate: true,
    statusPopup: null,
	uninitialized: false,
	
	isDisplayBottomButton: true,
    
    postCreate: function() {
        this.inherited('postCreate', arguments);

		//Set the max characters & absolute width hints on the editable title
		this.viewEditableTitle.editorParams = { maxLength: this.maxTitleLength };
        
        //Set the max characters & absolute width hints  on the editable summary
        this.viewEditableSummary.editorParams = { maxTextAreaLength: this.maxSummaryLength };
        this.viewEditableSummary.minWidthHint = 450;
        
        // server return status in hidden until a status is set
        //
        this.statusContainer.style.display = "none";
        
        this.statusSuccessNode.src = dojo.moduleUrl("com.ibm.asq.common.web", "ui/internal/view/layout/images/success_obj.gif");
        this.statusInfoNode.src = dojo.moduleUrl("com.ibm.asq.common.web", "ui/internal/view/layout/images/info_obj.gif");
        this.statusErrorNode.src = dojo.moduleUrl("com.ibm.asq.common.web", "ui/internal/view/layout/images/error_obj.gif");
        this.statusWarningNode.src = com.ibm.asq.common.web.ui.internal.view.common.ImageCache.WARNING_ICON;
        
		//bottom button area
		if(this.isDisplayBottomButton) {
			this.viewHeaderBottomNode.style.display = "";
			this.statusBottomContainer.style.display = "none";
			this.statusSuccessBottomNode.src = dojo.moduleUrl("com.ibm.asq.common.web", "ui/internal/view/layout/images/success_obj.gif");
        	this.statusInfoBottomNode.src = dojo.moduleUrl("com.ibm.asq.common.web", "ui/internal/view/layout/images/info_obj.gif");
        	this.statusErrorBottomNode.src = dojo.moduleUrl("com.ibm.asq.common.web", "ui/internal/view/layout/images/error_obj.gif");
        	this.statusWarningBottomNode.src = com.ibm.asq.common.web.ui.internal.view.common.ImageCache.WARNING_ICON;
		}
		
        this.setNoValueEditableTitleLabel(this.noTitleLabel);
        this.setNoValueEditableSummaryLabel(this.noSummaryLabel);
        
        this.setTitleEditable(this.titleEditable);
        this.setTitle(this.title);
        this.setSubTitle(this.subTitle);
        
        this.setTypeLabel(this.typeLabel);
        this.setIconClass(this.iconClass);
        this.setCSHelpId(this.contextHelpId, this.csHelpProviderId, this.csHelpEngine);

        this.setSummaryEditable(this.summaryEditable);
        this.setSummaryLabel(this.summaryLabel);
        this.setSummary(this.summary);

        // set up nodes for dual contents support
        this.typeNode.style.display = this.useDualContents ? "none" : "inline";
        this.contentSelectorNode.style.display = this.useDualContents ? "inline" : "none";

        if (this.useDualContents) {
            if (this.mainContentSelectedLabel == "")
                this.mainContentSelectedLabel = MessageUtil.getMessage(MessageBundle.COMMON, "sampleLabel_viewHeaderMainContentSelected");
            if (this.mainContentDeselectedLabel == "")
                this.mainContentDeselectedLabel = MessageUtil.getMessage(MessageBundle.COMMON, "sampleLabel_viewHeaderMainContentDeselected");
            if (this.altContentSelectedLabel == "")
                this.altContentSelectedLabel = MessageUtil.getMessage(MessageBundle.COMMON, "sampleLabel_viewHeaderAltContentSelected");
            if (this.altContentDeselectedLabel == "")
                this.altContentDeselectedLabel = MessageUtil.getMessage(MessageBundle.COMMON, "sampleLabel_viewHeaderAltContentDeselected");

            if (this.defaultMainContent) {
                this._selectMainContent();
            } else {
                this._selectAltContent();
            }
        }
        
        if( this.noline )
        {
        	this.viewHeaderBackNode.className = "view-header-back-no-line";
        }
		
		//set icon node to the right in RTL
		var lang = dojo.locale.split('-')[0];
		var RTL = {ar:true,fa:true,he:true,ur:true,yi:true,iw:true};
		if(RTL[lang]) {
			this.iconNode.style.cssFloat = "right";
			this.actionsAreaRight.style.cssFloat = "right";
			this.actionsAreaLeft.style.cssFloat = "right";
			this.viewHeaderRight.style.cssFloat = "right";
			this.wholestatusContainer.style.cssFloat = "left";
			this.statusContainer.style.cssFloat = "right";
			this.viewSummaryLabel.style.cssFloat = "right";
			
			this.actionsBottomAreaRight.style.cssFloat = "right";
			this.actionsBottomAreaLeft.style.cssFloat = "right";
			this.wholestatusBottomContainer.style.cssFloat = "left";
			this.statusBottomContainer.style.cssFloat = "right";
		}
		
		if (!dojo.isIE) {
			this.wholestatusBottomContainer.style.clear = "right";
		}
		
		if(dojo.isFunction(this.viewEditableTitle.edit)) {
			var self = this;
			this.viewEditableTitle.edit = function() {
				// set the minWidth for the edit box; need to
                // special case on IE to make it shorter. Otherwise,
                // it could push the inline edit box to the next line.
				var minWidth = self.viewHeaderRight.offsetLeft - this.domNode.offsetLeft;
                var offset = dojo.isIE ? 30 : 10;
				this.minWidthHint = minWidth - offset;
                
				this.inherited('edit', arguments);
			}
		}
        
        // store the view's right buttons for release later
        this.buttons = [];
    },

    setEditableTitleFocus: function()
    {
        // delay setting the editable title focus to let the widget load.
        setTimeout(dojo.hitch(this, function() {
            // we're setting the min width hint directly in the title's edit
            // method. See above...
            this.viewEditableTitle.focusWithPrompt();
        }), 30);
    },
    
    // set the no value prompt label for editable title
    //
    setNoValueEditableTitleLabel: function(label) {
        this.noTitleLabel = label;
        if (label) {
            this.viewEditableTitle.setNoValuePrompt(null, null, label);
        }
    },
    
    // change handler for editable title
    //
    editableTitleChange: function(value) {
		var val = dojo.trim( value );
		this.setTitle(val);
//		this.onTitleEdited(val);
    },
    
    setTitleEditable: function(editable) {
        this.titleEditable = editable;
        this.viewEditableTitle.domNode.style.display = this.titleEditable ? "inline" : "none";
        this.viewTitle.style.display = this.titleEditable ? "none" : "inline";
        this.viewSubTitle.style.display = this.titleEditable ? "none" : "inline";

        if (this.titleEditable) {
            this.viewEditableTitle.onChange = dojo.hitch(this, "editableTitleChange");			
			this.titleChangedHandle = dojo.connect(this.viewEditableTitle, "fireChangeEvent", this, "onTitleEdited");
            this.viewEditableTitle.onEditCallback = dojo.hitch(this, "hideCSHelp");
            this.viewEditableTitle.onEditCompleteCallback = dojo.hitch(this, "showCSHelp");
        }
    },

	uninitialize: function() {
		//console.debug("========uninitialize ViewHeaderLayout");
		this.uninitialized = true;
		var viewEditableTitle = this.viewEditableTitle;
		if (viewEditableTitle) {
			viewEditableTitle.onChange = null;
			viewEditableTitle.onEditCallback = null;
			viewEditableTitle.onEditCompleteCallback = null;
            if (this.titleChangedHandle) {
                dojo.disconnect(this.titleChangedHandle);
                delete this.titleChangedHandle;
            }
		}
		var viewEditableSummary = this.viewEditableSummary;
		if (viewEditableSummary) {
			viewEditableSummary.onChange = null;
            if (this.summaryChangedHandle) {
                dojo.disconnect(this.summaryChangedHandle);
                delete this.summaryChangedHandle;
            }
		}
        if(this.buttons) {
            dojo.forEach(this.buttons, function(pairButton){
               // If the buttons are just plain dom buttons,
               // need to clear its ref to the JS data.
               if(!pairButton.isButtonWidget) {
                   pairButton.headerButton.clientCallback = null;
                   pairButton.headerButton.callbackData = null;
                   pairButton.bottomButton.clientCallback = null;
                   pairButton.bottomButton.callbackData = null;
               } 
            });
        }
        if (this.statusPopup) {
            this.statusPopup.destroy();
        }
        if (this.statusBottomPopup) {
            this.statusBottomPopup.destroy();
        }
        if (this.dummyText) {
            this.dummyText.destroy();
        }
	},
	
    // set the title to display in the header
    //
    setTitle: function(title) {
        this.title = title;
        if (this.viewTitle)
            this.viewTitle.innerHTML = this.title ? this.title : "";
        if (this.viewEditableTitle) {
			this.viewEditableTitle.value = (this.title ? this.title : "");
			this.viewEditableTitle._setDisplayValue(this.title ? this.title : "");
		}
    },
	
	setSubTitle: function(subTitle) {
        this.subTitle = subTitle;
        if (this.viewSubTitle)
            this.viewSubTitle.innerHTML = this.subTitle ? "&nbsp;" + this.subTitle : "";
	},
    
    getTitle: function() {
        return this.title;
    },
    
    setTypeLabel: function(type) {
        this.typeLabel = type;
        if (this.typeNode)
            this.typeNode.innerHTML = this.typeLabel ? this.typeLabel : "";
    },
    
    setIconClass: function(iconClass) {
        this.iconClass = iconClass;
        if (this.iconNode) {
            this.iconNode.className = this.iconClass ? this.iconClass : "";
            this.iconNode.style.paddingRight = "10px";
        }
		if(this.iconClass)
			this.viewHeaderLeft.style.paddingLeft = "0px";
		else
			this.viewHeaderLeft.style.paddingLeft = "10px";
    },
    
    setCSHelpId: function(helpId, csHelpProviderId, csHelpEngine){
        this.contextHelpId = helpId;
        this.csHelpProviderId = csHelpProviderId;
        this.csHelpEngine = csHelpEngine;
        if (this.viewHelpButtonNode) {
            dojox.data.dom.removeChildren(this.viewHelpButtonNode);
            if (this.contextHelpId) {
                this.viewHelpButtonNode.style.display = "inline";
                var args = {
                    'contextID': helpId,
                    'csHelpEngine': this.csHelpEngine,
                    'csHelpProviderId': this.csHelpProviderId
                };
                this.cshelp = new com.ibm.asq.common.web.ui.internal.util.CSHelp(args);
                this.viewHelpButtonNode.appendChild(this.cshelp.domNode);
            } else {
                this.viewHelpButtonNode.style.display = "none";
            }
        }
    },
    
    hideCSHelp: function() {
        this.viewHelpButtonNode.style.display = "none";
    },
    
    showCSHelp: function() {
        if (this.contextHelpId) {
            this.viewHelpButtonNode.style.display = "inline";
        }
    },

	setCSHelpButtonLabel: function(/* REQUIRED */ cshelpLabel) {
		if(this.cshelp)
			this.cshelp.setButtonLabel(cshelpLabel);
	},

    // set the no value prompt label for editable title
    //
    setNoValueEditableSummaryLabel: function(label) {
        this.noSummaryLabel = label;
        if (!label) {
            if (this.summaryAsDesc) 
                label = MessageUtil.getMessage(MessageBundle.COMMON, "promptLabel_clickEnterDesc");
            else 
                label = MessageUtil.getMessage(MessageBundle.COMMON, "promptLabel_clickEnterSummary");
        }
        this.viewEditableSummary.setNoValuePrompt(null, null, label);
    },
    
    setSummaryEditable: function(editable) {
        this.summaryEditable = editable;
        this.viewEditableSummary.domNode.style.display = this.summaryEditable ? "inline" : "none";
        this.viewSummary.style.display = this.summaryEditable ? "none" : "inline";
        if (this.summaryEditable) {
            this.viewEditableSummary.onChange = dojo.hitch(this, "editableSummaryChange");
            this.summaryChangedHandle = dojo.connect(this.viewEditableSummary, "fireChangeEvent", this, "onSummaryEditedWrapper");
        }
    },

    editableSummaryChange: function(value) {
        // onChange handler when switching from edit mode to display mode
        this.setSummary(value);
        this.onSummaryEdited(value);
    },
    
    onSummaryEditedWrapper: function(value) {
        // handler for our "fireChangeEvent" on changes while still in edit mode
        this.setSummary(value, true);
        this.onSummaryEdited(value);
    },
    
    setSummaryLabel: function(label) {
        this.summaryLabel = label;
        this.viewSummaryLabel.innerHTML = label ? label : "";
        this.viewSummaryLabel.style.display = label ? "inline" : "none";
    },
    
    setSummary: function(summary, stayEdit) {
        this.summary = summary;
        if (this.viewSummary) {
            this.viewSummary.innerHTML = this.summary ? this.summary : "";
        }
        if (this.viewEditableSummary) {
            if (this.viewEditableSummary.editing && !stayEdit) {
                // cancel the editing unless it is explicitly set to stay in edit mode
                this.viewEditableSummary.cancel();
            }
            this.viewEditableSummary._setDisplayValue(this.summary ? this.summary : "");
            this.viewEditableSummary.value = this.summary ? this.summary : "";
        }
    },
    
    getSummary: function() {
        // the summary is updated by "fireChangeEvent" and onChange handlers
        return this.summary;
    },
    
    // Summary:
    //    set the status message in the header. Optionally, provide a type "Success", "Error",
    //    or "Info" parameter to qualify the status. 
    //
    setStatus: function(status, type, /*optional*/details) {
        this.statusContainer.style.display = "inline";
		this.statusBottomContainer.style.display = "inline";
        
        var stText = status;
        var detailText = details;
        
        // stop the status line from extending too far off to the
        // left. Cap its length at 40. Show the full text in the tooltip.
        //
        if((status.length - 40) > 5) {
            stText = status.substr(0, 40) + "...";
            // prefix the detail string with the status when the status
            // string is clipped. But only need to do that if the status
            // and the detail are different strings.
            if(details && details != status) {
                detailText = status + "<br />" + details;
            }
        }
        // detail message is in the tooltip popup
        if (this.statusPopup) {
            this.statusPopup.destroy();
        }
        this.statusPopup = new jazz.ui.HoverPopup({html: (details) ? detailText : status, around:[this.statusLineNode]});

        this.statusSuccessNode.style.display = (type == ViewHeaderLayout.SUCCESS) ? "inline" : "none";
        this.statusErrorNode.style.display = (type == ViewHeaderLayout.ERROR) ? "inline" : "none";
        this.statusInfoNode.style.display = "none";
        this.statusWarningNode.style.display = (type == ViewHeaderLayout.WARNING) ? "inline" : "none";
        
         this.statusLineNode.innerHTML = stText;
		 
		//bottom area
		this.statusContainer.style.display = "inline"; 
		this.statusBottomContainer.style.display = "inline";
		 // detail message is in the tooltip popup
        if (this.statusBottomPopup) {
            this.statusBottomPopup.destroy();
        }
        this.statusBottomPopup = new jazz.ui.HoverPopup({html: (details) ? detailText : status, around:[this.statusLineBottomNode]});
        this.statusSuccessBottomNode.style.display = this.statusSuccessNode.style.display;
        this.statusErrorBottomNode.style.display = this.statusErrorNode.style.display;
        this.statusInfoBottomNode.style.display = "none";
        this.statusWarningBottomNode.style.display = this.statusWarningNode.style.display;
        this.statusLineBottomNode.innerHTML = stText;
    },
   
    // set the status message in the header
    //
    clearStatus: function() {
        this.setStatus("");
        this.statusContainer.style.display = "none";
		this.statusBottomContainer.style.display = "none";
    },

    // Summary:
    //    show, or hide the content area DIV of the ViewHeaderLayout. 
    // 
    showContentArea: function(showContent /* true to show, false to hide */) {
        this.containerNode.style.display = (showContent) ? "inline" : "none";
    },
    
    // show the in-progress indicator and hide the title. Optionally, specify
    // whether the progress should hide the view's contents. If not specified, always shows
    // all contents. By hiding the view's content while the progress is showing,
    // this reduces the flickering while content is being loaded. When the progress is
    // ended all hidden content are shown.
    //
    setProgressStart: function(message, hideContent) {
        if (hideContent) {
            dojo.style(this.attributesContainerNode, "visibility", "hidden");
            dojo.style(this.typeNode, "visibility", "hidden");
            dojo.style(this.contentSelectorNode, "visibility", "hidden");
            dojo.style(this.summaryNode, "visibility", "hidden");
            dojo.style(this.actionsAreaRight, "visibility", "hidden");
            dojo.style(this.actionsAreaLeft, "visibility", "hidden");
            dojo.style(this.wholestatusContainer, "visibility", "hidden");
            
            this.progressHideContent = true;
			
			//bottom area
			dojo.style(this.actionsBottomAreaRight, "visibility", "hidden");
            dojo.style(this.wholestatusBottomContainer, "visibility", "hidden");
			dojo.style(this.actionsBottomAreaLeft, "visibility", "hidden");
        }
        this.progressLabel.innerHTML = message;
        this.viewHeaderLeft.style.display = "none";
        this.progressIndicator.style.display = "inline";
    },
    
    // restore the title and hide the in-progress indicator
    //
    setProgressEnd: function() {
        if(this.progressHideContent) {
            dojo.style(this.attributesContainerNode, "visibility", "visible");
            dojo.style(this.typeNode, "visibility", "visible");
            dojo.style(this.contentSelectorNode, "visibility", "visible");
            dojo.style(this.summaryNode, "visibility", "visible");
            dojo.style(this.actionsAreaRight, "visibility", "visible");
            dojo.style(this.actionsAreaLeft, "visibility", "visible");
            dojo.style(this.wholestatusContainer, "visibility", "visible");
			
			//bottom area
			dojo.style(this.actionsBottomAreaRight, "visibility", "visible");
            dojo.style(this.wholestatusBottomContainer, "visibility", "visible");
			dojo.style(this.actionsBottomAreaLeft, "visibility", "visible");
			
            this.progressHideContent = false;
        }
        this.progressLabel.innerHTML = "";
        this.viewHeaderLeft.style.display = "inline";
        this.progressIndicator.style.display = "none";
    },

    _addViewAction: function(label, handler, callbackData, iconClass, left, showActionLabel, dropDown) {
        var button = null;
		var arg = {
            label: label,
            showLabel: showActionLabel,
            'class': left ? "left-action" : "right-action",
            onClick: function( e ) {
                this.clientCallback(this.callbackData, e);
            }
        };
        
        if (iconClass) {
            arg["iconClass"] = iconClass;
        }
		if(dropDown) {
			arg['dropDown'] = dropDown;
			button = new dijit.form.DropDownButton(arg);
		}
		else 
        	button = new dijit.form.Button(arg);
        
        // store the client callback and data into the button
        button.clientCallback = handler;
        button.callbackData = callbackData;
        
        if (left)
            this.getleftActionsContainer().appendChild(button.domNode);
        else
            this.getRightActionsContainer().appendChild(button.domNode);
        
		//bottom button
		var bottomButton = null;
		if(dropDown) {
			bottomButton = new dijit.form.DropDownButton(arg);
		}
		else 
        	bottomButton = new dijit.form.Button(arg);
		bottomButton.clientCallback = handler;
        bottomButton.callbackData = callbackData;
		if (left)
            this.getleftActionsBottomContainer().appendChild(bottomButton.domNode);
        else
            this.getRightActionsBottomContainer().appendChild(bottomButton.domNode);
		
		var pairButtons = new HeaderLayout.PairButtons(button, bottomButton, true);	
        return pairButtons;
    },

    addLeftViewAction: function(label, handler, callbackData, iconClass, dropDown) {
        this.actionsAreaLeft.style.display = "block";
		this.actionsBottomAreaLeft.style.display = "block";
        return this._addViewAction(label, handler, callbackData, iconClass, true, false, dropDown);
    },
    
    addRightViewAction: function(label, handler, callbackData, iconClass) {
//        return this._addViewAction(label, handler, callbackData, iconClass, false, true);

//  use plain button at the moment
        var button = document.createElement('button');
        button.innerHTML = label;
        button.className = "right-action";
        this.connect(button, "onclick",  function( e ) { 
            e = e || window.event;
            e.target.focus();
            e.target.clientCallback(e.target.callbackData, e); 
		});
		this.connect(button, "onmousedown", dojo.stopEvent);
        button.tabIndex = "2";
		
        // store the client callback and data into the button
        // can't use dijit.connect for these as no dontFix arg is supported?
        button.clientCallback = handler;
        button.callbackData = callbackData;
        this.getRightActionsContainer().appendChild(button);
		
		//bottom area
        var bottomButton = button.cloneNode(true);
		this.connect(bottomButton, "onclick", function( e ) { 
            e = e || window.event;
            e.target.focus();
            e.target.clientCallback(e.target.callbackData, e); 
		});
		bottomButton.clientCallback = handler;
        bottomButton.callbackData = callbackData;
		this.connect(bottomButton, "onmousedown", dojo.stopEvent);
        this.getRightActionsBottomContainer().appendChild(bottomButton);
		
		var pairButtons = new HeaderLayout.PairButtons(button, bottomButton, false);
        
        // save the buttons, need to uninitialize the dom buttons
        // when the widget is destroy to break any circular refs.
        this.buttons.push(pairButtons);
        	
        return pairButtons;	
    },
    
    // return the right header DIV
    //
    getRightHeader: function() {
        return this.viewHeaderRight;
    },
    
    // return the main content area DIV
    //
    getContentNode: function() {
        return this.containerNode;
    },
    
    // return the alternate content area DIV
    //
    getAltContentNode: function() {
        return this.altContainerNode;
    },
    
    _selectMainContent: function() {
        if (this.useDualContents) {
            this.defaultMainContent = true;
            if (this.containerNode && this.altContainerNode) {
                this.mainContentSelector.innerHTML = this.mainContentSelectedLabel;
                this.mainContentSelector.className = "content-selector-selected";
                this.altContentSelector.innerHTML = this.altContentDeselectedLabel;
                this.altContentSelector.className = "content-selector-deselected";
                
                this.containerNode.style.display = "block";
                this.altContainerNode.style.display = "none";
				
				//set which content is selected
				this._mainContentSelected = true;
				this._altContentSelected = false;
            }
        }
    },
	
    isMainContentSelected: function() {
        return this._mainContentSelected;
    },	

    _selectAltContent: function() {
        if (this.useDualContents) {
            this.defaultMainContent = false;
            if (this.containerNode && this.altContainerNode) {
                this.mainContentSelector.innerHTML = this.mainContentDeselectedLabel;
                this.mainContentSelector.className = "content-selector-deselected";
                this.altContentSelector.innerHTML = this.altContentSelectedLabel;
                this.altContentSelector.className = "content-selector-selected";
                
                this.containerNode.style.display = "none";
                this.altContainerNode.style.display = "block";
				
				//set which content is selected
				this._altContentSelected = true;
                this._mainContentSelected = false;                			
            }
        }
    },

    isAltContentSelected: function() {
        return this._altContentSelected;
    },
	
    _onKeySelectMainContent: function(evt) {
        if(evt.keyCode == dojo.keys.ENTER || evt.charCode == dojo.keys.SPACE) {
            this._selectMainContent();
        }
    },
    
    _onKeySelectAltContent: function(evt) {
        if(evt.keyCode == dojo.keys.ENTER || evt.charCode == dojo.keys.SPACE) {
            this._selectAltContent();
        }
    },
    
    getInfoAttributesNode: function() {
        return this.attributesNode;
    },
    
    getExtInfoAttributesNode: function() {
        return this.attributesNodeExt;
    },
    
    addInfoLineBreak: function(/* optional */ position) {
        var br = document.createElement("span");
        br.innerHTML = "<br>";
        var childrenAttributes = this.attributesNode.childNodes;
        if (!position || (childrenAttributes.length < 1) || (position >= childrenAttributes.length)) {
            // append to the end if position is not specified or no children attribute
            // or position is out of range
            this.attributesNode.appendChild(br);
        } else {
            // insert before the child attribute of specified position
            this.attributesNode.insertBefore(br, childrenAttributes[position]);
        }
    },
    
    _createInfoAttributeNode: function(labelDomNode, valueDomNode) {
        var valueNode = valueDomNode;
        if( valueNode.nodeType == 3 )
        {
            //Cannot set the className on text nodes ( type 3 ) in IE
            valueNode = document.createElement( "span" );
            valueNode.appendChild( valueDomNode );
        }
        
        var attributeContainer = document.createElement("span");
        attributeContainer.className = "attributes-entry";
        dojo.addClass(labelDomNode, "attributes-entry-label");
        dojo.addClass(valueNode, "attributes-entry-value");
       
        attributeContainer.appendChild(labelDomNode);
        attributeContainer.appendChild(valueNode);
        return attributeContainer;
    },
    
    addInfoAttribute: function(labelDomNode, valueDomNode, /* optional */ position) {
        var attributeContainer = this._createInfoAttributeNode(labelDomNode, valueDomNode);
        var childrenAttributes = this.attributesNode.childNodes;
        if (!position || (childrenAttributes.length < 1) || (position >= childrenAttributes.length)) {
            // append to the end if position is not specified or no children attribute
            // or position is out of range
            this.attributesNode.appendChild(attributeContainer);
        } else {
            // insert before the child attribute of specified position
            this.attributesNode.insertBefore(attributeContainer, childrenAttributes[position]);
        }
		return attributeContainer;
    },
    
    setExtInfoAttribute: function(labelDomNode, valueDomNode) {
        var attributeContainer = this._createInfoAttributeNode(labelDomNode, valueDomNode);
        dojox.data.dom.removeChildren(this.attributesNodeExt);
        this.attributesNodeExt.appendChild(attributeContainer);
        return attributeContainer;
    },
    
    getSummaryNode: function() {
        return this.summaryNode;
    },

    getleftActionsContainer: function() {
        return this.actionsAreaLeft;
    },

    getRightActionsContainer: function() {
        return this.actionsAreaRight;
    },
	
	getleftActionsBottomContainer: function() {
        return this.actionsBottomAreaLeft;
    },

    getRightActionsBottomContainer: function() {
        return this.actionsBottomAreaRight;
    },
    
    onTitleEdited: function(value) {
        // called when the title (editable) has been modified by the user.    
    },

    onSummaryEdited: function(value) {
        // called when the summary (editable) has been modified by the user.    
    }
});

//	Create some definitions that will be used in other files
//	Note that the ViewHeaderLayout variable is used locally to this file as a shortcut.
	var ViewHeaderLayout = com.ibm.asq.common.web.ui.internal.view.layout.ViewHeaderLayout;
	ViewHeaderLayout.ERROR 		= 1;
	ViewHeaderLayout.SUCCESS 	= 2;
	ViewHeaderLayout.INFO 	= 3;
	ViewHeaderLayout.WARNING	= 4;
})();