/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008, 2009. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
 // Summary:
 //   The TableCellRenderer is an "interface" object that the TableViewer uses
 //   to draw a cell in the table. Specific cell renderer must be subclassed from this
 //   interface class and implements all public methods defined in this class. The
 //   cell renderer, employed by the TableViewer, is responsible for drawing a cell in 
 //   a table row. Specific subclasses will render the cell data in the intended display
 //   style. For example, a basic text cell renderer will draw the text data in the cell,
 //   or a image cell renderer will draw an image that represents the data in the cell, etc.
 //   More complex cell renderer could allow the user to change the value of the cell
 //   data. It's the responsibility of the renderer to send the value changed event to
 //   the supplied cell modifier.
 //
dojo.provide("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableCellRenderer");

dojo.require("com.ibm.asq.common.web.ui.internal.util.AutoWordWrapping");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableConstants");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableCellModifier");
dojo.require("jazz.ui.HoverPopup");

(function() {

var WordWrappingUtil = com.ibm.asq.common.web.ui.internal.util.wordWrapping;

dojo.declare("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableCellRenderer", null,
{   
    constructor: function()
    {
        this.cellNodes = [];
        this.connections = [];    
    },
    
    // Summary:
    //    Renders the cell content. The specific subclass will draw the appropriate
    //    widget for the cell rendition. 
    //    @param viewer the viewer that uses this cell renderer
    //    @param cellData The data to be displayed. The data can be an object to hold sufficient data for 
    //    the appropriate renderer type (obtained from the table's content provider)
    //    @param extendedDisplayInfo Additional display info about the cell.
    //    @param cellModifier The modifier object that can receive value changed events from the cell.
    //    @param rowObject The data object for the row. It's the client data that represent the row
    //    @param columnIndex  the column index of the cell
    //    @param cellWidthHint optional parameter to indicate the width of the cell, which the cell content *should not* excess
    //    @return the domNode for the cell content.
    //
    renderCell: function(viewer, cellData, extendedDisplayInfo, cellModifier, rowObject, columnIndex, /*optional*/cellWidthHint)
    {
        // Subclass must provide a dom node to display the cell data.
        // In addition, if the cellWidthHint parameter is provided, the cell
        // renderer should render the content within that width boundry. Use
        // this base class's _clipText method for clipping the textual contents.
        //
        return null;
    },
	
	// Summary:
	//    Create a unique label for the cell according to the row and column
	getCellLabel: function(viewer, rowObject, columnIndex) 
	{
		var contentProvider = viewer.getContentProvider();
		var rowName = (contentProvider.getRowDataName && dojo.isFunction(contentProvider.getRowDataName)) ? contentProvider.getRowDataName(rowObject) : null;
		var columnName = contentProvider.getColumnHeading(viewer.getContent(), columnIndex);
		rowName = (rowName) ? rowName : (rowObject.name || rowObject.id);
		var label = "column: " + columnName + " " + ((rowName) ? "row:" + rowName : "");
		return label;
	},

    
    // Summary:
    //   Attaches a tooltip to the specified node. If the specified tooltipText is null,
    //   or is empty, no tooltip will be created for the node.
    //
    _attachTooltipToCell: function(node, tooltipText)
    {
        var tooltip = null;
        
        if(tooltipText && tooltipText.length > 0)
            tooltip = new jazz.ui.HoverPopup({html: tooltipText, around:[node]});
        return tooltip;
    },
    
    // Summary:
    //    Clips the text at the specified width. The text will be clipped and an ellipsis is
    //    added to the end of the text.
    //
    ClipTextClassName: "clip-cell-nowrap", // copied in TableViewer!!!
    _clipText: function(viewer, text, maxWidth)
    {
    	return {'clipped': false, 'text': text};
    },
    
    // Summary:
    //    Class method to handle compact cell mode in the table viewer. All subclasses should use
    //    this method to configure a textual display node to handle text clipping and the proper
    //    style sheet settings for compact cell.
    //
    _handleCompactCellText: function(viewer, text, extendedDisplayInfo, node, maxWidth)
    {
		var ttText = text;
        if (extendedDisplayInfo && extendedDisplayInfo['tooltip']) {
            ttText = text + "<br></br>" + extendedDisplayInfo['tooltip'];
        }
        
        var ret = (dojo.isString(text) && text.length > 0) ? text : " ";
        dojo.addClass(node, this.ClipTextClassName);
        if (ret.length >4){ 
	    	if (dojo.isFF){
	    		node.style.cssText='width: '+maxWidth+'px; '+
	    			'-moz-binding: url("/jazz/web/com.ibm.asq.common.web/ui/internal/widgets/templates/tableViewer/ellipsis.xml#ellipsis");';
			}else{
				node.style.width=''+maxWidth+'px';
			}
    	}
        node.title=text;	// use default tooltip as since we do not know if it is clipping we always need it 
        return ret;
    },
    
    // Summary:
    //    Class method to render text in a table cell. When possible, always use this method
    //    to draw text in a table cell. This method properly adjusts the text being displayed
    //    to conform to the table's compact settings and preserves the client's character in the
    //    display. This method expects a BLOCK dom node (at least a 'div') for the display node.
    //
    _renderCellText: function(viewer, cellData, extendedDisplayInfo, node, cellWidthHint)
    {
        var text = (cellData == null || (dojo.isString(cellData) && cellData.length == 0)) ? "" : cellData + "";
        
        if(viewer.isCompactMode && typeof cellWidthHint != "undefined") {
            text = this._handleCompactCellText(viewer, text, extendedDisplayInfo, node, cellWidthHint);
        }
        else {
            if(text.length == 0)
                text = " ";
            
            if (extendedDisplayInfo && dojo.isObject(extendedDisplayInfo)) 
                node.tooltip = this._attachTooltipToCell(node, extendedDisplayInfo['tooltip']);
        }
        
        // wrap the text with a text node to ensure all characters are displayed. 
        // If not, HTML tag characters like '<' could be escaped.
        var textNode = document.createTextNode(text);
        node.appendChild(textNode);
        
    },
    
    // Summary:
    //    Internal method to connect cell node's event to an internal function. 
    //    All cell subclass should use this method to connect cell node's events
    //    to internal functions. 
    // 
    connect: function(node, event, func)
    {
        var cnt = dojo.connect(node, event, this, func);  
        this.connections.push(cnt);  
    },

    // Summary:
    //    This method is called by the table viewer when the viewer is destroyed, or 
    //    a page of table data is removed. All cell renderer should clear any dom node 
    //    to JS object references here.
    //
    disconnectAll: function()
    {
        var len = this.connections.length;
        var d = dojo;
        for(var i = 0; i < len; i++) {
            d.disconnect(this.connections[i]);
        }
        this.connections = [];
    }
    
});

})();
